/*	EP.EXE		Sun and moon EPHEMERIS calculator.
 */

#include <dos.h>
#include <errno.h>
#include <conio.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <io.h>
#include <sys\types.h>
#include <sys\stat.h>

#include "ephheadr.h"

/* Globals */

   int mn,yr;
   double dy;
   int hr,m,s;
   double lam,bet,hp;
   double phase,el;     
   double lsn,rsn,ehp;
   double deps,dpsi;
   double ra,dec;
   double ha;
   double aha,adec;
   double alt,az;
   double pressure,temp;
   double mjd,mjd_day,epoch_mjd;
   double utc,gst;
   double lst;
   double longitude,latitude,height;  

int its_dark = FALSE;
int sun_is_down = FALSE;
int moon_is_down = FALSE;
double sunset, moonset;

char line[ 80 ], filespath[80];
struct  dosdate_t dt;	/* date of exposure */
struct  dostime_t tm;	/* time of exposure */

Starstruc  Catalog[] = {
"Alpha Persei",1.79,"Mirphak",    3.3933,49.8167,1989.5,0.0,0.0,
"Alpha Tau A",  .85,"Aldebaran",  4.5883,16.4883,1989.5,0.0,0.0,
"Beta Ori A",   .12,"Rigel",      5.2333,-8.2167,1989.5,0.0,0.0,
"Alpha Aur AB", .08,"Capella",    5.2650,45.9833,1989.5,0.0,0.0,
"Beta Tau",    1.65,"Alnath",     5.4283,28.6000,1989.5,0.0,0.0,
"Alpha Ori",    .5, "Betelgeuse", 5.9100, 7.4000,1989.5,0.0,0.0,
"Beta Aur",    1.90,"Menkalinan", 5.9800,44.9500,1989.5,0.0,0.0,
"Gamma Gem",   1.93,"Alhena",     6.6183,16.4167,1989.5,0.0,0.0,
"Alpha CMa A",-1.46,"Sirius",    6.7450,-16.7000,1989.5,0.0,0.0,
"Alpha Gem A", 1.94,"Castor",     7.5650,31.9167,1989.5,0.0,0.0,
"Alpha CMi A",  .38,"Procyon",    7.6467, 5.2500,1989.5,0.0,0.0,
"Beta Gem",    1.14,"Pollux",     7.7450,28.0500,1989.5,0.0,0.0,
"Alpha Leo A", 1.35,"Regulus",   10.1300,12.0167,1989.5,0.0,0.0,
"Alpha UMa AB",1.79,"Dubhe",     11.0517,61.8000,1989.5,0.0,0.0,
"Epsilon UMa", 1.8, "Alioth",    12.8933,56.0167,1989.5,0.0,0.0,
"Alpha Vir",   1.0, "Spica",    13.4100,-11.1000,1989.5,0.0,0.0,
"Eta UMa",     1.86,"Alkaid",    13.7850,49.3667,1989.5,0.0,0.0,
"Alpha Boo",   -.04,"Arcturus",  14.2533,19.2333,1989.5,0.0,0.0,
"Alpha Lyr",    .03,"Vega",      18.6100,38.7667,1989.5,0.0,0.0,
"Alpha Aql",    .77,"Altair",    19.8383, 8.8333,1989.5,0.0,0.0,
"Alpha Cyg",   1.25,"Deneb",     20.6850,45.2500,1989.5,0.0,0.0
};

/* Function prototypes used in this program. */

void _cdecl main ( int, char *[] );
void init_ep ( void );
void init_struct ( void );
void show_usage( void );
void do_startup ( char * );
char *get_nxt_tok( FILE * );
int get_nxt_number ( FILE * );
double get_double ( FILE * );
int strmatch ( char *, char * );
void do_sunpos( void );
void do_moonpos( void );
void set_date ( char * );
void set_time ( char * );

/* Display usage of KI */

void show_usage( void )
{
  printf("\nUsage: EP [-Start filename1]\n\n"
  	 "  where: '-Start filename1	Name of EP start/config file.\n" );
  exit ( 1 );
}

void _cdecl main( int argc, char *argv[] )
{
	int i, fhandle;
	char	startfile[100], startflname[50];
	
	_dos_getdate( &dt );
	_dos_gettime( &tm );
	
	init_ep();

	/* Process command line args */
	
	line[0] = startflname[0] = '\0';
	if ( *argv[1] == '?' )
	   show_usage();
	   	
	for ( i=1; i<argc; i++ )
	{
	    if ( argv[i][0] == '-' )
	    {
		if ( strmatch( &argv[i][1], "start" ) )
			strcpy ( startflname, argv[ ++i ] );
		else if ( strmatch( &argv[i][1], "date" ) )
		{
			set_date( argv[ ++i ] );
		}	
		else if ( strmatch( &argv[i][1], "time" ) )
		{
			set_time( argv[ ++i ] );
		}	
		else if ( strmatch ( &argv[i][1], "longitude" ) )
			longitude = (double) atof( argv[ ++i ] );
		else if ( strmatch ( &argv[i][1], "latitude" ) )
			latitude = (double) atof( argv[ ++i ] );
		else if ( strmatch ( &argv[i][1], "height" ) )
			height = (double) atof( argv[ ++i ] );
		else if ( strmatch( &argv[i][1], "sunset" ) )
			sunset = (double) atof( argv[ ++i ] );
		else if ( strmatch ( &argv[i][1], "moonset" ) )
			moonset = (double) atof( argv[ ++i ] );
		else if ( strmatch ( &argv[i][1], "pressure" ) )
			pressure = (double) atof( argv[ ++i ] );
		else if ( strmatch ( &argv[i][1], "temp" ) )
			temp = (double) atof( argv[ ++i ] );
		else
		{
			printf ( "ERROR: Unknown keyword '%s' command line\n",
				argv[i] );
			exit ( 1 );
		}
	    }
	}
	
	/* See if startup file has been specfied. */

	if ( startflname[0] )
	{
		strcpy ( startfile, filespath );
		strcat ( startfile, startflname );
		do_startup( startfile );	/* Process startup file. */
	}


	/* Finally, do the work. */
	 
   yr = dt.year;  
   mn = dt.month;
   dy = (double) dt.day;
   hr = tm.hour;
   m  = tm.minute;
   s  = tm.second; 

	printf("INPUT DATA: %4d/%2d/%2d  %2d:%2d:%2d\n", 
		dt.year, dt.month, dt.day,
		tm.hour, tm.minute, tm.second );
	printf("Latitude: %7.3f  Longitude: %7.3f  Height: %7.3f\n", 
   		latitude, longitude, height );
	printf("Pressure: %7.3f  Temperature: %7.3f\n", pressure, temp );

   sex_dec(hr,m,s,&utc);                      /* convert UT to decimal repr. */
   cal_mjd(mn,dy,yr,&mjd_day);                /* return modified Julian date */
   mjd = mjd_day + (utc/24);            /* add UT time in fractions of a day */
   printf("Julian Date %f \n",mjd+2415020.0);
   utc_gst(mjd_day,utc,&gst);     /* find greenwich mean sidereal time, *gst */
   lst =gst + deghr(longitude);              /* now find local sidereal time */
   range(&lst,24.0);                                  /* keep it within 24 h */
   printf("Local Sidereal Time: %f\n",lst);
   latitude =degrad(latitude);                /* convert latitude to radians */
   height /=6378140.0;                /* heigh as a fraction of earth radius */

	do_sunpos();
	do_moonpos();
/*	do_starpos();*/
printf("sun = %d moon = %d\n", sun_is_down, moon_is_down);

if ( sun_is_down == TRUE )
   printf("Sun is down\n");
if ( moon_is_down == TRUE )
   printf("Moon is down\n");
if ( moon_is_down && sun_is_down ){
   printf("It's dark\n");
   fhandle = open("RUNKI", O_RDWR|O_CREAT|O_TRUNC|O_TEXT, S_IREAD|S_IWRITE);
   close( fhandle );
}
}

void init_ep ( void )	/* Initialize data fields. */
{
  char *env;

  
  /* Get environment variables for path names. */

  if ( ( env = getenv( "KI_FILES" ) ) != (char *) NULL )
  {
     strcpy( filespath, env );		/* Base path for images. */
     strcat( filespath, "\\" );
  }
  else
     filespath[0] = '\0';

     longitude  = -114.1313889;
     latitude   = 51.080000000;
     height     = 1115.4; 
     sunset	= -10.0;
     moonset	= 0.0;
     pressure   = 1000.0; 		/*mBar*/
     temp       = -10.0; 			/* Celsius */
}

/* Process EP startup file. 
 *
 * Input is in the form: -keyword [keyvalue] [keyvalue]
 * where keywords may have zero or more keyvalues to use.
 */

void do_startup ( char *startfilename )
{
FILE 	*f1;
char	*token;

	if ( ( f1 = fopen ( startfilename, "r" ) ) == (FILE *) NULL )
	{
		printf( "ERROR: Unable to open startup file: %s\n",
			startfilename );
		exit ( -1 );
	}
	
	while ( 1 )
	{
	    token = get_nxt_tok ( f1 );

	    if ( token == (void *) NULL )
	    	break;
	    else if ( token[0] == '-' )		/* Keyword */
	    {
		if ( strmatch( &token[1], "date" ) )
		{
			token = get_nxt_tok ( f1 );
			set_date( token );
		}	
		else if ( strmatch( &token[1], "time" ) )
		{
			token = get_nxt_tok ( f1 );
			set_time( token );
		}
		else if ( strmatch ( &token[1], "longitude" ) )
			longitude = get_double( f1 );
		else if ( strmatch ( &token[1], "latitude" ) )
			latitude = get_double( f1 );
		else if ( strmatch ( &token[1], "height" ) )
			height = get_double( f1 );
		else if ( strmatch( &token[1], "sunset" ) )
			sunset = get_double( f1 );
		else if ( strmatch ( &token[1], "moonset" ) )
			moonset = get_double( f1 );
		else if ( strmatch ( &token[1], "pressure" ) )
			pressure = get_double( f1 );
		else if ( strmatch ( &token[1], "temp" ) )
			temp = get_double( f1 );
		else
		{
			printf ( "ERROR: Unknown keyword '%s' in startup file '%s'\n",
				token,  startfilename );
			exit ( 1 );
		}
	    }
	    else
	    {
		printf ( "ERROR: Unknown keyword '%s' in startup file '%s'\n",
			token,  startfilename );
		exit ( 1 );
	    }
	}
	fclose ( f1 );
}

/* Compare first 'n' chars of in_token to target.
 *
 * Return true if they match, false otherwise. 
 */
  
int strmatch ( char *in_token, char *target )
{
register int i, j;

	j = strlen ( in_token );
	if ( j )
	{
		for ( i=0; i<j; i++ )
		{
			if ( !target[ i ] )
				return 0;   /* in_token longer than target */
			if ( !( target[ i ] == in_token[ i ] ) )
				return 0;   /* No compare */
		}
	}
	else
		return 0;		    /* NULL string */
		
	return 1;
}

/* Get next token from startup file.
 * Return pointer to stringified token, or NULL if EOF.
 */
 
char *get_nxt_tok( FILE *infile )
{
static char	outstring[80], instring[100] = "\n";
static int	x = 0;
int		i, stopper;

	/* Get a line of input from the startup file. */

	while ( instring[x] == '\n' )
	{
		if ( fgets ( instring, 100, infile ) == (void *) NULL )
			return( (void *) NULL );
		x = 0;
	}
	
	/* Skip over white space to next token. */
	
	while ( 	(instring[x] == ' ' ) 
		     || (instring[x] == '\t' ) )
		x++;

	/* Copy next token to output string. */

	i = 0;
	if ( instring[x] == '"' )
	{
		stopper = '"';
		x++;
	}
	else
		stopper = ' ';
		
	while ( 1 )
	{
		if (	(instring[x] == (char) stopper ) 
		     || (instring[x] == '\t' ) 
		     || (instring[x] == '\n' ) )
		     break;
		else
			outstring[ i++ ] = instring[ x++ ];
		if ( i == 80 )
			break;
	}

	outstring[ i ] = '\0';

	return strlwr ( outstring );
}


/* Get next token from startup file and return as numeric integer value. */

int get_nxt_number ( FILE *infile )
{
char	*token;
int	i, j;

	token = get_nxt_tok( infile );
	j = strlen ( token );
	for ( i=0; i<j; i++ )
	if ( !((token[i] == '-') || ((token[i] >= '0') && (token[i] <= '9' ))))
	{
		printf("ERROR: Numeric value expected.\n");
		exit( -1 );
	}
	return ( atoi ( token ) );	
}

/* Get next token from startup file, return as double precision real value. */

double get_double ( FILE *infile )
{
char	*token;
int	i, j;
double	retval;

	token = get_nxt_tok( infile );
	j = strlen ( token );
	for ( i=0; i<j; i++ )
	if ( !(	(token[i] == '-') 
		|| ( token[i] == '+' )
		|| ( token[i] == '.' )
		|| ((token[i] >= '0') && (token[i] <= '9' ))))
	{
		printf("ERROR: floating point value expected.\n");
		exit( -1 );
	}
	sscanf( token, "%f", &retval );
	return ( retval );	
}

void do_sunpos( void )
{

   sunpos(mjd,&lsn,&rsn);                   /* return true ecl long and dist */
   nutation(mjd,&deps,&dpsi);     /* deps: nutat in obl, dpsi: nutat in long */
   lsn += dpsi;                      /* correct longitude (lam) for nutation */
   lsn -= degrad(20.4/3600);                            /* light travel time */
   ecl_eq(mjd,0.0,lsn,&ra,&dec);             /* convert to equatorial coords */
   range(&lst,24.0);                                  /* keep it within 24 h */
   ha =(hrrad(lst)) - ra;                                 /* find hour angle */
   ehp = (2.0 * 6378.0 / 146.0e6) / rsn;
   ta_par(ha,dec,latitude,height,ehp,&aha,&adec); /* correct for geoc parllx */
   hadec_aa(latitude,aha,adec,&alt,&az);        /* convert to horizon coords */
   refract(pressure,temp,alt,&alt);               /* correct for refraction. */
   
   printf("\nSOLAR EPHEMERIS:\n");
   printf("-------------------------------------\n");
   printf("S:Ecliptic Longitude: %f, Ecliptic Latitude: 0.0 \n",raddeg(lsn));
   printf("S:Right Ascension: %f, Declination: %f \n",radhr(ra),raddeg(dec));
   printf("S:Hour Angle: %f, Altitude: %f, Azimuth: %f\n",
          radhr(ha), raddeg(alt), raddeg(az));
   if ( raddeg(alt) < sunset )
      sun_is_down = TRUE;
   else
      sun_is_down = FALSE;
}

void do_moonpos( void )
{

   moon(mjd,&lam,&bet,&hp); /* return geocentric ecl long, lat, hor parallax */
   nutation(mjd,&deps,&dpsi);     /* deps: nutat in obl, dpsi: nutat in long */
   lam += dpsi;                      /* correct longitude (lam) for nutation */
   range(&lam,2*PI);
   ecl_eq(mjd,bet,lam,&ra,&dec);             /* convert to equatorial coords */
   ha =(hrrad(lst)) - ra;                                 /* find hour angle */
   ta_par(ha,dec,latitude,height,hp,&aha,&adec);  /* correct for geoc parllx */
   hadec_aa(latitude,aha,adec,&alt,&az);        /* convert to horizon coords */
   refract(pressure,temp,alt,&alt);               /* correct for refraction. */

   printf("\nLUNAR EPHEMERIS:\n");
   printf("-------------------------------------\n");
   printf("M:Ecliptic Longitude: %f, Ecliptic Latitude: %f \n",
          raddeg(lam),raddeg(bet));
   printf("M:Right Ascension: %f, Declination: %f \n",radhr(ra),raddeg(dec));
   printf("M:Hour Angle: %f, Altitude: %f, Azimuth: %f\n",
          radhr(ha),raddeg(alt),raddeg(az));
   sunpos(mjd,&lsn,&rsn);     /* lsn: solar ecl long */
   range(&lsn,2*PI);
   elongation(lam,bet,lsn,&el);
   phase = (double) fabs(el)/PI*100.0;
   printf("M:Phase: %f percent.\n",phase);
   if ( raddeg(alt) < moonset )
      moon_is_down = TRUE;
   else
      moon_is_down = FALSE;
}

/* Set date from 'date_str' arg.  Format YYMMDD. */

void set_date ( char *date_str )
{
char	str2[3];
int	val;

	str2[0] = date_str[0];
	str2[1] = date_str[1];
	str2[2] = '\0';
	val = atoi( str2 );
	if ( val < 50 ) 
		val += 2000;	/* Ready for next century!! */
	else
		val += 1900;	
	dt.year = val;
	str2[0] = date_str[2];
	str2[1] = date_str[3];
	dt.month = (unsigned char) atoi( str2 );
	str2[0] = date_str[4];
	str2[1] = date_str[5];
	dt.day = (unsigned char) atoi( str2 );
}

/* Set time from 'time_str' arg.  Format HHMMSS. */

void set_time ( char *time_str )
{
char	str2[3];

	str2[0] = time_str[0];
	str2[1] = time_str[1];
	str2[2] = '\0';
	tm.hour = (unsigned char) atoi( str2 );
	str2[0] = time_str[2];
	str2[1] = time_str[3];
	tm.minute = (unsigned char) atoi( str2 );
	str2[0] = time_str[4];
	str2[1] = time_str[5];
	tm.second = (unsigned char) atoi( str2 );
}

#ifdef SKIP
void do_starpos( void )
{
   int i;
   printf("\n");
   printf("BRIGHT STAR CATALOG\n");
   printf("===================\n");
   printf("        Name     R.A.   Decl.  Azimuth  Zenith Angle \n");
   printf("--------------------------------------------------------\n");
   for(i =0; i <= NUMBER_OF_STARS-1; i +=1) {
     ra =hrrad(Catalog[i].RA);                             /* Right Asc. */
     dec =degrad(Catalog[i].DEC);                         /* Declination */
     year_mjd(Catalog[i].Epoch,&epoch_mjd);         /* find mjd of epoch */
     precess(epoch_mjd,mjd,&ra,&dec);                    /* precess star */
     ha =hrrad(lst) - ra;                             /* find hour angle */
     hadec_aa(latitude,ha,dec,&alt,&az);    /* convert to horizon coords */
     refract(pressure,temp,alt,&alt);          /* correct for refraction */
     Catalog[i].AZ =raddeg(az);
     Catalog[i].ZenithAng = 90 - raddeg(alt);   /* Zenith Angle = 90-alt */
     printf("%12s %7.1f %7.1f",Catalog[i].PopName,Catalog[i].RA,Catalog[i].DEC);
     printf(" %7.1f %7.1f\n",Catalog[i].AZ,Catalog[i].ZenithAng);
   }
/*printf("\n");*/
}
#endif
